import os

def verify_django():
    if 'DJANGO_SETTINGS_MODULE' in os.environ:
        settings = __import__(os.environ['DJANGO_SETTINGS_MODULE']).settings
    else:
        raise ImportError("Django工程未被执行，请在Django工程中导入Sqlee的Django组件.")
    try:
        if not settings.ENABLE_SQLEE or settings.SQLEE_NAME.replace(" ", "") == "":
            raise ImportError('Sqlee未在Django被正确配置.')
    except Exception as exc:
        raise ImportError('Sqlee未在Django被正确配置.') from exc



def model_to_data(model=None):
    return tuple([i.data if hasattr(i, "data") else i for i in model])

class SqleeException(Exception):
	"""
        Sqlee Base Exception.
    """

class SqleeWarning(Warning):
    """
        Sqlee Base Warning.
    """

class VersionError(SqleeException):
    pass

class RepositoryNotFound(SqleeWarning):
    def __init__(self, repo_name=None, args="目标数据库 '{}' 不存在，将尝试建立它."):
        if repo_name:
            args = args.format(repo_name)
        super(RepositoryNotFound, self).__init__(args)
        self.repo_name = repo_name

class RepositoryNotFoundError(SqleeException):
    def __init__(self, repo_name=None, args="目标数据库 '{}' 不存在."):
        if repo_name:
            args = args.format(repo_name)
        super(RepositoryNotFoundError, self).__init__(args)
        self.repo_name = repo_name

class RepositoryConnectionRefusedError(SqleeException, ConnectionRefusedError):
    def __init__(self, repo_name=None, status_code=None, args="连接目标数据库 '{}' 出现异常."):
        if repo_name:
            args = args.format(repo_name)
        if status_code:
            args += " [Code %d]" % status_code
        super(RepositoryConnectionRefusedError, self).__init__(args)
        self.repo_name = repo_name
        self.status_code = status_code

class TableNotFoundError(SqleeException):
    def __init__(self, table_name=None, args="目标表 '{}' 不存在."):
        if table_name:
            args = args.format(table_name)
        super(TableNotFoundError, self).__init__(args)
        self.table_name = table_name

class ColumnNotFoundError(SqleeException):
    def __init__(self, column_name=None, args="目标表 '{}' 不存在."):
        if column_name:
            args = args.format(column_name)
        super(ColumnNotFoundError, self).__init__(args)
        self.column_name = column_name

class ColumnDataDoesNotFit(SqleeWarning):
    def __init__(self, namespace=None, datas=None):
        args = "数据错误: 捕获到的数据与命名域不符:\n\t命名域: {}\n\t数据: {}."
        args = args.format(namespace, model_to_data(datas))
        super(ColumnDataDoesNotFit, self).__init__(args)

class DataWasNotGiven(SqleeWarning):
    def __init__(self, namespace=None, args="数据错误: 数据 '{}' 未被传入，使用空值代替."):
        super(DataWasNotGiven, self).__init__(args.format(namespace))

class DataWasNotGivenError(SqleeException):
    def __init__(self, namespace=None, args="数据错误: 数据 '{}' 未被传入."):
        super(DataWasNotGivenError, self).__init__(args.format(namespace))

class InternetError(SqleeException):
    def __init__(self):
        super(InternetError, self).__init__("网络连接异常，请检查网络连接.")


#coding: utf-8

from sqlee.utils.backend import SqleeTable
from sqlee.utils.backend import table_objects

import os
import json
import requests

if __name__ == "__main__":
    from verify import verify_django
else:
    from .verify import verify_django

class Model:
    def __init__(self):
        self.repo = settings.SQLEE.repo
        self.tablename = "%s_%s" % (
            self.app,
            self.__class__.__name__
            )
        try:
            self.table = SqleeTable(name=self.tablename, repo=self.repo)
            self.objects = table_objects(obj=self.table)
        except Exception:
            pass

verify_django()


#coding: utf-8
"""
   作者: Entropy
   版本: 1.0
"""
from urllib.parse import urlencode

import requests
import json
import base64

if __name__ == "__main__":
    from utils.exceptions import RepositoryNotFoundError, RepositoryConnectionRefusedError, InternetError
else:
    from .exceptions import RepositoryNotFoundError, RepositoryConnectionRefusedError, InternetError, ColumnNotFoundError

class GiteeRepo:
    token = None
    user = None
    repo = None
    def __init__(self, token=None, user=None, repo=None):
        self.token = token
        self.user = user
        self.repo = repo
        self.url = "https://gitee.com/" + user + "/" + repo + "/"

        try:
            query = requests.get(
                url = "https://gitee.com/api/v5/user/repos",
                data = {"access_token": token}
                )
            all_repos = query.json()
            exists = False
            for repos in all_repos:
                if repo == repos["name"] and user == repos["owner"]["login"]:
                    exists = True
            if not exists:
                raise RepositoryNotFoundError(self.repo)
            if query.status_code // 100 != 2:
                raise RepositoryConnectionRefusedError(self.repo, status_code=status_code)
        except requests.RequestException:
            raise InternetError()

        self.types = {}
        self.types["int"] = int
        self.types["str"] = str

    def get_file(self, path=None):
        query = requests.get(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?access_token={self.token}"
            ).text
        if query == "[]":
            raise ColumnNotFoundError(path)
        return base64.b64decode(json.loads(query)["content"].encode()).decode()

    def get_data(self, path=None):
        return json.loads(base64.b64decode(json.loads(requests.get(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?access_token={self.token}"
            ).text)["content"].encode()).decode())["content"]

    def list_file(self, path=None, detail=False, int=False, dir=False):
        if detail and int:
            print("[-] 参数'detail'与参数'int'不可以均设置为True.")
            return False
        result = []
        reception = json.loads(requests.get(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?access_token={self.token}"
            ).text)
        for file in reception:
            if detail:
                if not dir:
                    if file["type"] != "dir":
                        result.append(file)
                else:
                    result.append(file)
                continue
                
            if not int:
                if not dir:
                    if file["type"] != "dir" and file["name"] != ".keep" and file["name"] != ".namespace":
                        result.append(file["name"])
                else:
                    if file["name"] != ".keep" and file["name"] != ".namespace":
                        result.append(file["name"])
            else:
                if file["name"] != ".keep" and file["name"] != ".namespace":
                    if not dir:
                        if file["type"] != "dir":
                            try:
                                result.append(self.types["int"](file["name"]))
                            except ValueError:
                                raise ValueError("目标文件名不支持转换为数字.")
                    else:
                        try:
                            result.append(self.types["int"](file["name"]))
                        except ValueError:
                            raise ValueError("目标文件名不支持转换为数字.")
        return tuple(result)

    def list_file_int(self, path=None):
        return self.list_file(path=path, detail=False, int=True, dir=False)

    def list_folder(self, path=None, detail=False, int=False):
        if detail and int:
            print("[-] 参数'detail'与参数'int'不可以均设置为True.")
            return False
        result = []
        reception = json.loads(requests.get(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?access_token={self.token}"
            ).text)
        for file in reception:
            if detail:
                if file["type"] == "dir":
                    result.append(file)
                continue

            if file["type"] == "dir":
                if not int:
                    result.append(file["name"])
                else:
                    try:
                        result.append(self.types["int"](file["name"]))
                    except ValueError:
                        raise ValueError("目标文件夹名不支持转换为数字.")
        return tuple(result)

    def list_dir(self, path=None, detail=False, int=False):
        return self.list_folder(path=path, detail=detail, int=int)
    
    def list_folder_int(self, path=None):
        result = []
        reception = json.loads(requests.get(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?access_token={self.token}"
            ).text)
        for file in reception:
            if file["type"] == "dir":
                result.append(self.types["int"](file["name"]))
        return tuple(result)

    def upload_file(self, path=None, content=None):
        return requests.post(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}",
            data = {
                "access_token": self.token,
                "content": base64.b64encode(content.encode()),
                "message": "UPLOAD FILE - SQLEE"
                }
            )

    def update_file(self, path=None, content=None):
        return requests.put(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?access_token={self.token}",
            data = {
                "access_token": self.token,
                "content": base64.b64encode(content.encode()),
                "sha": self.get_sha(path=path),
                "message": "UPDATE FILE - SQLEE"
                }
            )

    def delete_file(self, path=None):
        data = {
            "access_token": self.token,
            "sha": self.get_sha(path=path),
            "message": "DELETE FILE - SQLEE"
            }
        return requests.delete(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?" + urlencode(data)
            )

    def get_sha(self, path=None):
        result = json.loads(requests.get(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}?access_token={self.token}"
            ).text)
        if isinstance(result, dict):
            if "sha" in result:
                return result["sha"]
            else:
                raise ValueError("捕获到的数据无法提取文件sha值.")
        else:
            print(result)
            raise ValueError("捕获到的数据无法提取文件sha值.")

    def make_folder(self, path=None):
        return requests.post(
            url = f"https://gitee.com/api/v5/repos/{self.user}/{self.repo}/contents/{path}/.keep",
            data = {
                "access_token": self.token,
                "content": base64.b64encode("*".encode()),
                "message": "MAKE FOLDER - SQLEE"
                }
            )

    def drop_folder(self, path=None):
        for folder in self.list_file(path=path, detail=True, dir=True):
            if folder["type"] == "dir":
                self.drop_folder(path=folder["path"])
            elif folder["type"] == "file":
                self.delete_file(path=folder["path"])
        return True
        
    def __str__(self):
        return self.repo

def make_repo(token=None, user=None, name=None, private=True, auto_init=True):
    if not token or not user or not name:
        raise ValueError("参数不全.")
    return requests.post(
        url = "https://gitee.com/api/v5/user/repos",
        data = {
            'name': name,
            'access_token': token,
            'auto_init': auto_init,
            'private': private,
        }
    )
if __name__ == "__main__":
    pass
    #db = GiteeRepo(token="1895956f770eb0e4d08013ee4b753203", user="fu050409", repo="TEST_API")
    #db.drop_folder(path="Table")
    #print(gitee.get_file("TEST"))
    #print(gitee.list_file(""))
    #print(gitee.list_folder("Table", int=True))
    #print(gitee.list_folder("Table", int=False, detail=True))
    #print(gitee.upload_file(path="TEST_UPLOAD", content="测试上传文件"))
    #print(gitee.delete_file(path="TEST_UPLOAD"))
    #print(gitee.update_file(path="TEST_COMMITTER", content="测试更新文件"))
